@@ -24,7 +24,11 @@ require_relative 'generic'
 module Async
 	module IO
 		class Stream
-			def initialize(io, block_size: 1024*8, sync: true)
+			# The default block size for IO buffers.
+			# BLOCK_SIZE = ENV.fetch('BLOCK_SIZE', 1024*16).to_i
+			BLOCK_SIZE = 1024*8
+			
+			def initialize(io, block_size: BLOCK_SIZE, sync: true)
 				@io = io
 				@eof = false
 				
@@ -38,27 +42,68 @@ module Async
 			end
 			
 			attr :io
+			attr :block_size
 			
 			# Reads `size` bytes from the stream. If size is not specified, read until end of file.
 			def read(size = nil)
-				return "" if size == 0
+				return '' if size == 0
 				
-				until @eof || (size && size <= @read_buffer.size)
-					fill_read_buffer
+				if size
+					fill_read_buffer until @eof or @read_buffer.size >= size
+				else
+					fill_read_buffer until @eof
 				end
-
+				
 				return consume_read_buffer(size)
 			end
-
+			
+			# Read at most `size` bytes from the stream. Will avoid reading from the underlying stream if possible.
+			def read_partial(size = nil)
+				return '' if size == 0
+				
+				if @read_buffer.empty? and !@eof
+					fill_read_buffer
+				end
+				
+				return consume_read_buffer(size)
+			end
+			
+			# Efficiently read data from the stream until encountering pattern.
+			# @param pattern [String] The pattern to match.
+			# @return [String] The contents of the stream up until the pattern, which is consumed but not returned.
+			def read_until(pattern, offset = 0)
+				until index = @read_buffer.index(pattern, offset)
+					offset = @read_buffer.size
+					
+					return unless fill_read_buffer
+				end
+				
+				matched = @read_buffer.slice!(0, index)
+				@read_buffer.slice!(0, pattern.bytesize)
+				
+				return matched
+			end
+			
+			def peek
+				until yield(@read_buffer) || @eof
+					fill_read_buffer
+				end
+			end
+			
 			# Writes `string` to the buffer. When the buffer is full or #sync is true the
 			# buffer is flushed to the underlying `io`.
 			# @param string the string to write to the buffer.
 			# @return the number of bytes appended to the buffer.
 			def write(string)
-				@write_buffer << string
-				
-				if @write_buffer.size > @block_size
-					flush
+				if @write_buffer.empty? and string.bytesize >= @block_size
+					syswrite(string)
+				else
+					@write_buffer << string
+					
+					if @write_buffer.size >= @block_size
+						syswrite(@write_buffer)
+						@write_buffer.clear
+					end
 				end
 				
 				return string.bytesize
@@ -73,8 +118,10 @@ module Async
 
 			# Flushes buffered data to the stream.
 			def flush
-				syswrite(@write_buffer)
-				@write_buffer.clear
+				unless @write_buffer.empty?
+					syswrite(@write_buffer)
+					@write_buffer.clear
+				end
 			end
 
 			def gets(separator = $/)
@@ -93,8 +140,8 @@ module Async
 
 			# Closes the stream and flushes any unwritten data.
 			def close
-				flush rescue nil
-				
+				flush
+			ensure
 				@io.close
 			end
 
@@ -107,46 +154,26 @@ module Async
 			
 			alias eof eof?
 			
-			# Efficiently read data from the stream until encountering pattern.
-			# @param pattern [String] The pattern to match.
-			# @return [String] The contents of the stream up until the pattern, which is consumed but not returned.
-			def read_until(pattern)
-				index = @read_buffer.index(pattern)
-				
-				until index
-					offset = @read_buffer.size
-
-					fill_read_buffer
-					
-					return if @eof
-
-					index = @read_buffer.index(pattern, offset)
-				end
-				
-				matched = @read_buffer.slice!(0, index)
-				@read_buffer.slice!(0, pattern.bytesize)
-				
-				return matched
-			end
-			
-			def peek
-				until yield(@read_buffer) || @eof
-					fill_read_buffer
-				end
-			end
-			
 			private
 			
 			# Fills the buffer from the underlying stream.
 			def fill_read_buffer
-				if buffer = @io.read(@block_size)
-					@read_buffer << buffer
-				else
-					@eof = true
+				# Can we read directly into the buffer? (Ruby doesn't support append, only replace):
+				if @read_buffer.empty?
+					if @io.read(@block_size, @read_buffer)
+						return true
+					end
+				elsif chunk = @io.read(@block_size)
+					@read_buffer << chunk
+					return true
 				end
+				
+				# We didn't read anything, so we must be at eof:
+				@eof = true
+				return false
 			end
 
-			# Consumes `size` bytes from the buffer.
+			# Consumes at most `size` bytes from the buffer.
 			# @param size [Integer|nil] The amount of data to consume. If nil, consume entire buffer.
 			def consume_read_buffer(size = nil)
 				# If we are at eof, and the read buffer is empty, we can't consume anything.
@@ -154,7 +181,7 @@ module Async
 				
 				result = nil
 				
-				if size == nil || size == @read_buffer.size
+				if size == nil || size >= @read_buffer.size
 					# Consume the entire read buffer:
 					result = @read_buffer.dup
 					@read_buffer.clear
