@@ -23,14 +23,12 @@ require_relative 'socket'
 module Async
 	module IO
 		class UNIXSocket < BasicSocket
-			wraps ::UNIXSocket, :path, :addr, :peeraddr
-			
-			# TODO Currently unimplemented.
-			# , :send_io, :recv_io
+			# `send_io`, `recv_io` and `recvfrom` may block but no non-blocking implementation available.
+			wraps ::UNIXSocket, :path, :addr, :peeraddr, :send_io, :recv_io, :recvfrom
 		end
 		
 		class UNIXServer < UNIXSocket
-			wraps ::UNIXServer
+			wraps ::UNIXServer, :listen
 			
 			def accept
 				peer = async_send(:accept_nonblock)
@@ -44,6 +42,9 @@ module Async
 					wrapper.close
 				end
 			end
+			
+			alias sysaccept accept
+			alias accept_nonblock accept
 		end
 	end
 end
